#!/usr/bin/perl -W
#
# @file
# Perl based xapian indexer
#
# Development by Lachlan Gunn - http://drupal.org/user/277696
# Polishing and release by Simon Lindsay - http://drupal.org/user/143
#

use strict;
use Search::Xapian;
use DBI;

my $search_db = Search::Xapian::WritableDatabase->new('xapian_database', Search::Xapian::DB_CREATE_OR_OPEN);

my $indexer = Search::Xapian::TermGenerator->new();
my $stemmer = Search::Xapian::Stem->new("english");
$indexer->set_stemmer($stemmer);

# database, username, password
my $db = DBI->connect('dbi:mysql:database', 'username', 'password') || die $DBI::errstr;

my $node_statement = $db->prepare(<<SQL);
    SELECT
      node_revisions.nid,
      node_revisions.body,
      node.type,
      node.title

    FROM
      node_revisions

    INNER JOIN
      node
      ON
        node.nid = node_revisions.nid

    ORDER BY
      node.nid ASC
SQL

my $taxonomy_statement = $db->prepare(<<SQL);
   SELECT
     node.nid,
     term_data.name

   FROM
     node

   LEFT JOIN
     term_node
     ON
       term_node.nid = node.nid

   LEFT JOIN
     term_data
     ON
       term_data.tid = term_node.tid

   ORDER BY
     node.nid ASC
SQL

my $count_statement =
  $db->prepare("SELECT COUNT(*) FROM node");

$count_statement->execute();
(my $nodes) = $count_statement->fetchrow_array();

print STDERR "Executing database queries...";
$node_statement->execute();
$taxonomy_statement->execute();
print STDERR "Done.\n";

my $row_number = 0;
my $temp_counter = 0;
my $time = time();

my @taxonomy_row = $taxonomy_statement->fetchrow_array();

$search_db->begin_transaction();
while (my @row = $node_statement->fetchrow_array) {
  my $nid = $row[0];
  my $body = $row[1];
  my $type = $row[2];
  my $title = $row[3];

  $body =~ s/<\w+>//g;
  $body =~ s/\s+/ /g;

  $row_number++;
  $temp_counter++;

  my $current_time = time();
  my $delta = $current_time - $time;

  # Only commit to database every 30 records
  if ($delta >= 30) {
    $search_db->commit_transaction();
    $search_db->begin_transaction();

    my $percent = int(100*100*$row_number / $nodes) / 100;

    my $rate = $temp_counter / $delta;
    my $remaining = int(($nodes - $row_number) / $rate / 60.0);
    printf STDERR ("Indexing node %d of %d (%d%%), %d minutes remaining, %d nodes/second\n",
                   $row_number, $nodes, $percent, $remaining, $rate);

    $temp_counter = 0;
    $time = $current_time;
  }

  my $document = Search::Xapian::Document->new();

  $indexer->set_document($document);

  $indexer->index_text($title, 5);
  $indexer->index_text($body);
  
  $document->set_data($nid);
  $document->add_term('Q'.$nid);
  $document->add_term('XNODETYPE'.$type);

  while ($taxonomy_row[0] < $nid) {
    @taxonomy_row = $taxonomy_statement->fetchrow_array();
  }

  if ($taxonomy_row[0] == $nid) {
    do {
      my $term = $taxonomy_row[1];
      if (defined($term)) {
        $document->add_term('XTAXONOMY'.lc($term));
        $indexer->index_text($term, 2);
      }

      @taxonomy_row = $taxonomy_statement->fetchrow_array();
    } while (@taxonomy_row && defined($taxonomy_row[0]) && $taxonomy_row[0] == $nid);
  }

  $search_db->add_document($document);
}
$search_db->commit_transaction();
